﻿using System;

using NUnit.Framework;

using Org.BouncyCastle.Asn1.Pkcs;
using Org.BouncyCastle.Crypto;
using Org.BouncyCastle.Crypto.Generators;
using Org.BouncyCastle.Crypto.Parameters;
using Org.BouncyCastle.Math;
using Org.BouncyCastle.Security;
using Org.BouncyCastle.Utilities.Test;

namespace Org.BouncyCastle.Pkcs.Tests
{
    [TestFixture]
    public class EncryptedPrivateKeyInfoTest
        : ITest
    {
        private readonly string alg = PkcsObjectIdentifiers.PbeWithShaAnd3KeyTripleDesCbc.Id; // 3 key triple DES with SHA-1

		public string Name
        {
			get { return "EncryptedPrivateKeyInfoTest"; }
        }

		public ITestResult Perform()
        {
            try
            {
                IAsymmetricCipherKeyPairGenerator pGen = GeneratorUtilities.GetKeyPairGenerator("RSA");
                RsaKeyGenerationParameters genParam = new RsaKeyGenerationParameters(
					BigInteger.ValueOf(0x10001), new SecureRandom(), 512, 25);

				pGen.Init(genParam);

				AsymmetricCipherKeyPair pair = pGen.GenerateKeyPair();

                //
                // set up the parameters
                //
                byte[] salt = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };
                int iterationCount = 100;

				//
                // set up the key
                //
                char[] password1 = { 'h', 'e', 'l', 'l', 'o' };

                EncryptedPrivateKeyInfo  encInfo = EncryptedPrivateKeyInfoFactory.CreateEncryptedPrivateKeyInfo(alg, password1, salt, iterationCount, PrivateKeyInfoFactory.CreatePrivateKeyInfo(pair.Private));

                PrivateKeyInfo info = PrivateKeyInfoFactory.CreatePrivateKeyInfo(password1, encInfo);

                AsymmetricKeyParameter key = PrivateKeyFactory.CreateKey(info);

                if (!key.Equals(pair.Private))
                {
                    return new SimpleTestResult(false, Name + ": key corrupted.");
                }

                return new SimpleTestResult(true, Name + ": Okay");
            }
            catch (Exception e)
            {
                return new SimpleTestResult(false, Name + ": exception - " + e.Message);
            }
        }

        public static void Main(
            string[] args)
        {
            ITest test = new EncryptedPrivateKeyInfoTest();
            ITestResult result = test.Perform();

			Console.WriteLine(result);
        }

		[Test]
        public void TestFunction()
        {
            string resultText = Perform().ToString();

            Assert.AreEqual(Name + ": Okay", resultText);
        }
    }
}
